// DEEP WEB TOUR -----------
// Canvas Drawing System
// 3am LLC 2014
// -------------------------

// Global Variables
var cvs;
var ctx;
var strokes = [];

if(typeof window.devicePixelRatio === "undefined")
	window.devicePixelRatio = 1.0;

// Constants
var dc = {
	maxRes: 800,		// Maximum resulution of the canvas
	smooth: 1,			// Amount of smoothness to apply
	preserve: 0.015,	// Preserve points that cause a significant angular change
	sdsamples: 40		// Maximum number of points the shape detector should consider
};

var style = {
	color: "red",
	width: 10
}
var drawInit = false;
// Start the canvas
function THREEAM_DRAWING(){

	// Fetch globals
	cvs = document.getElementById("drawingCanvas");
	ctx = cvs.getContext("2d");
	
	// Clear the stored data
	strokes = [];
	
	// Resize the canvas space appropriately
	var scale = resizeCanvas();

	// Touch start
	var drawing = false;
	var points = [];

	// MOUSE & TOUCH EVENTS
	// Start off the interactions
	if(!drawInit)
	{
		drawInit = true;
		bindElement(cvs,"mousedown touchstart",function(e){
			e.preventDefault();
			drawing = true;
			ctx.fillStyle = style.color;
	
			var nx,ny;
			nx = e.clientX*window.devicePixelRatio;
			ny = e.clientY*window.devicePixelRatio;
			
			// Handle Touch Events
			if(e.changedTouches){
				nx = e.changedTouches[0].pageX*window.devicePixelRatio;
				ny = e.changedTouches[0].pageY*window.devicePixelRatio;
			}
			
			ny -= cvs.getBoundingClientRect().top*window.devicePixelRatio;
			nx -= cvs.getBoundingClientRect().left*window.devicePixelRatio;
	
			points = [[nx*scale,ny*scale]];
			addBrushPoint(nx*scale,ny*scale,ctx,false);
		});
	
		bindElement(cvs,"mousemove touchmove",function(e){
			e.preventDefault();
			if(!drawing)return;
			
			var nx,ny;
			nx = e.clientX*window.devicePixelRatio;
			ny = e.clientY*window.devicePixelRatio;
			
			// Handle Touch Events
			if(e.changedTouches){
				nx = e.changedTouches[0].pageX*window.devicePixelRatio;
				ny = e.changedTouches[0].pageY*window.devicePixelRatio;
			}
			
			ny -= cvs.getBoundingClientRect().top*window.devicePixelRatio;
			nx -= cvs.getBoundingClientRect().left*window.devicePixelRatio;
	
			points.push([nx*scale,ny*scale]);
			addBrushPoint(nx*scale,ny*scale,ctx,true);
		});
	
		bindElement(cvs,"mouseup touchend mouseleave touchleave",function(e){
			e.preventDefault();
			if(!drawing)return;
			drawing = false;
			// Save the points to the stroke history
			savePoints(points, style, strokes);
		});
		
		// DOM MANAGEMENT
		// Interacts with the HTML
		
		// Size the canvas appropriately
		function resizeCanvas(){
		
			// Do the math
			var aspect = cvs.offsetWidth/cvs.offsetHeight;
			var hres = Math.min(cvs.offsetWidth*window.devicePixelRatio, dc.maxRes);
		
			// Set the values
			cvs.setAttribute('width',hres);
			cvs.setAttribute('height',(hres/aspect));
		
			// Create the back buffer
			startBuffer(cvs);
		
			// Return the scale factor
			return hres / (cvs.offsetWidth*window.devicePixelRatio);
		
		}
		window.addEventListener("resize",resizeCanvas);
	}
}



// Helper function so I don't have to add jQuery
function bindElement(el, s, fn) {
	var evts = s.split(' ');
	for (var i=0, iLen=evts.length; i<iLen; i++) {
		el.addEventListener(evts[i], fn, false);
	}
}

// LIVE BITMAP DRAWING
// Allows for live drawing using simple linearly extrapolated points

var lbx, lby;
function addBrushPoint(x,y,ctx,continueLine){

	// Calculate distance moved where possible
	var dist = 0;
	if(!lbx || !lby){
		continueLine = false;
	}else{
		dist = distance([x,y],[lbx,lby]);
	}
	
	ctx.beginPath();
	if(!continueLine || dist < style.width/2) {
		// Distance is small enough that a circle will do
		ctx.arc(x,y,style.width*window.devicePixelRatio/2,0,2*Math.PI);
		ctx.fill();
	}else{
		// Draw a pill-like shape to stand in for missing pixels
		ctx.lineCap="round";
		ctx.moveTo(lbx,lby);
		ctx.lineTo(x,y);
		ctx.lineWidth = style.width*window.devicePixelRatio;
		ctx.strokeStyle = ctx.fillStyle;
		ctx.stroke();
	}

	lbx = x; lby = y;
}

function savePoints(points,style,strokes){

	// Run the curve cleaner
	loadFromBuffer(ctx);
	curveCleaner(points,ctx);
	saveToBuffer(cvs);

	// Add the points to the strokes vector
	strokes.push({
		p: points,
		c: style.color,
		w: style.width
	});
}

function exportStrokes(){
	// Convert all points to integers
	for(i in strokes){for(b in strokes[i].p){strokes[i].p[b][0]=Math.round(strokes[i].p[b][0]);strokes[i].p[b][1]=Math.round(strokes[i].p[b][1]);}}

	// Return JSON
	return "bbs"+JSON.stringify(strokes);
}

// GEOMETRY FUNCTIONS
// I knew math would come in handy eventually!

function square(x) { return x * x }
function distance(A,B){
	return Math.sqrt(square(A[0] - B[0]) + square(A[1] - B[1]));
}
function angleCompare(a,b){
	diff = Math.abs(a-b);
	if(diff < Math.PI)return diff;
	return (2*Math.PI)-diff;
}

// Figures out how much 'offset' point B causes from the A-C line
function pointBOffsetDistance(A,B,C){

	// Get the length of line AC
	var lineLength = distance(A,C);
	if(lineLength == 0)return distance(B,A);

	// Basically a dot product, for vector mapping
	var unit = ((B[0] - A[0]) * (C[0] - A[0]) + (B[1] - A[1]) * (C[1] - A[1])) / square(lineLength);
	if (unit < 0) return distance(B, A);
  	if (unit > 1) return distance(B, C);

  	// Return the calculated distance
  	return distance(B, [ A[0] + unit * (C[0] - A[0]), A[1] + unit * (C[1] - A[1]) ]);
}

function pointBAngleOffset(A,B,C){

	// Get angles
	var aba = Math.atan2(A[0]-B[0],A[1]-B[1]);
	var aca = Math.atan2(A[0]-C[0],A[1]-C[1]);
	var diff = Math.abs(aca - aba);

	if(diff < Math.PI)return diff;
	return (2*Math.PI)-diff;
}

// SMOOTHED VECTOR DRAWING
// The second drawing engine, smoothes point drawings into clean vectors

// Clean-curve drawing function
function curveCleaner(points, ctx){

	// PHASE 1: Remove Unnecessary Points
	var reduced = 0;
	reduced += reducePoints(points);
	var reductionPasses = Math.min(2,parseInt(points.length/300));
    for(var i = 0; i < reductionPasses; i++){
        reduced += reducePoints(points);
    }

	// PHASE 2: Draw Points Quadratically
	drawPoints(points,ctx);	

}

function reducePoints(points){
	var reduced = 0;
	for(var i = 0; i < points.length - 2; i++){
		var bd = pointBOffsetDistance(points[i],points[i+1],points[i+2]);
		var ba = pointBAngleOffset(points[i],points[i+1],points[i+2]);

		if(bd < dc.smooth && ba < dc.preserve){
			points.splice(i+1,1); reduced++;
		}
	}
	return reduced;
}

function drawPoints(points,ctx){
	// Only draw 1 point
	if(points.length == 1){
		addBrushPoint(points[0][0],points[0][1],ctx,false);
		return;
	}

	// Draw curved line
	ctx.beginPath();
	ctx.lineCap="round";
	ctx.lineJoin = "round";

	ctx.moveTo(points[0][0],points[0][1]);
	var i;
	for(i = 1; i < points.length - 2; i++){
		var xc = (points[i][0] + points[i + 1][0]) / 2;
      	var yc = (points[i][1] + points[i + 1][1]) / 2;
		ctx.quadraticCurveTo(points[i][0], points[i][1], xc, yc);
	}
	if(points.length > 2){
		ctx.quadraticCurveTo(points[i][0], points[i][1], points[i+1][0],points[i+1][1]);
	}else if(points.length == 2){
		ctx.lineTo(points[1][0],points[1][1]);
	}

	ctx.stroke();
}

// STATE MANAGEMENT (UNDO)
// Since the drawing is stored as a vector, we can undo!

function undoToState(state){

	// Start from scratch
	ctx.clearRect(0,0,10000,10000);

	// Redraw strokes
	state = Math.min(state, strokes.length);
	for(var i = 0; i < state; i++){
		ctx.fillStyle = strokes[i].c;
		ctx.strokeStyle = strokes[i].c;
		ctx.lineWidth = strokes[i].w;
		drawPoints(strokes[i].p,ctx);
	}

	// Save the new starting state to the buffer
	saveToBuffer(cvs);

	// Remove later strokes entirely
	strokes.splice(state,strokes.length-state);
}

function undo(){undoToState(strokes.length-1);}


// MANAGE BACK-BUFFER
// Used to separate live drawings from smoothed drawings

var backCanvas;
var backContext;

// Create a new buffer
function startBuffer(canvas){
	backCanvas = document.createElement('canvas');
	backCanvas.width = canvas.width;
	backCanvas.height = canvas.height;
	backContext = backCanvas.getContext('2d');
}

// Save canvas to buffer
function saveToBuffer(canvas){
	backContext.clearRect(0,0,10000,10000);
	backContext.drawImage(canvas, 0,0);
}

// Load buffer into canvas
function loadFromBuffer(ctx){
	ctx.clearRect(0,0,10000,10000);
	ctx.drawImage(backCanvas, 0,0);
}
